@@ -1,6 +1,6 @@
 #include "SWFWriter.h"
-#include <string.h>
-#include <stdlib.h>
+#include <cstring>
+#include <cstdlib>
 
 namespace SWF {
 
@@ -58,6 +58,50 @@ void Writer::putInt64( uint64_t value ) {
 	putInt( value & 0xffffffff );
 }
 
+void Writer::putU30( uint32_t value ) {
+	byteAlign();
+		
+	int bytes = 1;
+	uint32_t limit = 0x80;
+	for(; value > limit - 1; limit *= 0x80) {
+		bytes++;
+	}
+
+	if( !assure( bytes ) ) return;
+	// FIXME x86-centric?
+	
+	for(; bytes > 0; bytes--) {
+		data[pos] = value & 0x7F;
+		
+		if(bytes > 1) {
+			value = value >> 7;
+			data[pos] |= 0x80;
+		}
+		pos++;
+	}
+}
+
+void Writer::putS24( int value ) {
+	byteAlign();
+	if( !assure( 3 ) ) return;
+	// FIXME x86-centric?
+	value = (unsigned int)value;
+	data[pos++] = value&0xFF;
+	data[pos++] = (value>>8)&0xFF;
+	data[pos++] = (value>>16)&0xFF;
+}
+
+void Writer::putEncodedU32(unsigned int value) {
+	for (size_t i = 0; i < 5; ++i) {
+		if (value < 0x80) {
+			data[pos++] = value;
+			return;
+		}
+		data[pos++] = (value & 0x7f) | 0x80;
+		value >>= 7;
+	}
+}
+
 void Writer::putFloat( float v ) {
     /*
 	byteAlign();
@@ -70,21 +114,71 @@ void Writer::putFloat( float v ) {
 	data[pos++] = (value>>16)&0xFF;
 	data[pos++] = (value>>24)&0xFF;
     */
-    union {
-        double f;
-        uint32_t ul;
-    } u;
-    u.f = v;
-    putInt(u.ul);
+	union {
+		float f;
+		uint32_t ul;
+	} u;
+	u.f = v;
+	putInt(u.ul);
 }
 
 void Writer::putDouble( double v ) {
-    union {
-        double d;
-        uint64_t ull;
-    } u;
-    u.d = v;
-    putInt64(u.ull);
+	union {
+		double d;
+		uint64_t ull;
+	} u;
+	u.d = v;
+	putInt64(u.ull);
+}
+
+void Writer::putDouble2( double v ) {
+	union {
+		double d;
+		char c[8];
+	} u;
+	u.d = v;
+	for( int i=0; i < 8; i++ ) {
+		data[pos++] = u.c[i];
+	}
+}
+
+void Writer::putHalf( float v ) {
+	union {
+		float f;
+		uint32_t ul;
+	} u;
+	u.f = v;
+
+	int sign = u.ul >> 31;
+	int exp = (u.ul >> 23) & 0xFF;
+	int man = (u.ul & 0x007FFFFF);
+
+	if( exp == 0 ) {
+		man = 0;
+	} else if( exp == 0xFF ) {
+		exp = 0x1F;
+		if( man != 0 ) {
+			man = 1;
+		}
+	} else {
+		exp -= 127;
+		if( exp < -14 ) {
+			if( exp >= -24 ) {
+				man = (man | 0x800000) >> (-exp - 14 + 13);
+			} else {
+				man = 0;
+			}
+			exp = 0;
+		} else if( exp > 15 ) {
+			exp = 0x1F;
+			man = 0;
+		} else {
+			exp += 15;
+			man >>= 13;
+		}
+	}
+
+	putWord((sign << 15) | (exp << 10) | man);
 }
 
 void Writer::putFixed( double value, int bytesize, int exp ) {
@@ -116,6 +210,15 @@ void Writer::putPString( const char *value ) {
 	pos += len;
 }
 
+void Writer::putPStringU30( const char *value ) {
+	byteAlign();
+	int len = strlen(value);
+	if( !assure( len+1 ) ) return;
+	putU30(len);
+	memcpy( &data[pos], value, len );
+	pos += len;
+}
+
 void Writer::putNBitInt( int64_t value, int n, bool is_signed ) {
 	// FIXME: unefficient, maybe incorrect!
 	if( !assure( ((n+bits)/8)+1 ) ) return;
